/*
********************************************************************
(c) MSU Video Group 2003-2006, http://compression.ru/video/
This source code is property of MSU Graphics and Media Lab

This code may be distributed under LGPL
(see http://www.gnu.org/licenses/lgpl.html for more details).

E-mail: video-measure@compression.ru
Author: Oleg Petrov
********************************************************************
*/  

/**
*  \file icustomplugin.h
*  \brief More convenient interface for metric development.
*/

#pragma once

#include <vector>
#include <string>

using namespace std;

#include "imetricplugin.h"
#include "imetricimage.h"

/*!\brief Convenient interface for metric development
*
*	General interface for metric is IMetricPlugin,
*	but this interface (ICustomPlugin) is much more convenient.
*	ICustomPlugin is adapted to IMetricPlugin through CPluginAdapter.
*	So to create metric, you need to do:
*		1. Create CMyMetric:ICustomPlugin
*		2. In exported function CreateMetric() write
*			
*			ICustomPlugin *my_plugin = new CMyMetric;
*			*metric = new CPluginAdapter(my_plugin);
*
*/
class ICustomPlugin
{
public:

	virtual ~ICustomPlugin() {};	//!< Destructor


	/**
	**************************************************************************
	* \brief Initialize metric plugin
	*
	*	Use this function to initialize metric parameters.
	*	It is always called before Measure(), MeasureAndVisualize(), GetAverage(), ResetMetricStat(),
	*	but other functions may be called earlier than Init().
	*	If metric supports configuration string, then Config() is called after Init().
	*	global_id represents current available global_id for metric.
	*	If metric uses N id, it can increment global_it by N, reserving
	*	[global_id ... (global_id + (N-1))] for its' use. This reserved IDs will identify
	*	results of metric in Measure(), MeasureAndVisualize(), MapIDToFrame() and so on.
	*
	* \param cc			[IN] - current color component, selected by user.
	* \param width		[IN] - width of images that will be measured.
	* \param height		[IN] - height of images that will be measured.
	* \param global_id	[IN,OUT] - currently available global identifier.
	*						Metric may reserve a number of identifiers by incrementing this value.
	*/
	virtual void Init(IMetricPlugin::COLORCOMPONENT cc, int width, int height, int &global_id) = 0;

	/**
	**************************************************************************
	* \brief Measures metric on images
	*
	* \param images			[IN] - vector of images, as many as metric needs (it is returned by GetVideoNum(false)).
	* \return vector of pairs "id, value corresponding to the ID".
	*/
	virtual vector< pair <IMetricPlugin::ID, float> >	Measure(vector<IMetricImage*> &images) = 0;

	/**
	**************************************************************************
	* \brief Measures metric on images and saves visualization
	*
	* \param images			[IN] - vector of images, as many as metric needs (it is returned by GetVideoNum(true)).
	* \param vis			[IN, OUT] - pointer to RGB24 image, stored as BGRBGRBGRBGR...
	* \param vis_pitch		[IN] - row pitch of visualization image: distance in bytes between beginning of rows.
	* \return vector of pairs "id, value corresponding to the ID".
	*/
	virtual vector< pair <IMetricPlugin::ID, float> >	MeasureAndVisualize(vector<IMetricImage*>&images, BYTE *vis, int vis_pitch) = 0;

	/**
	**************************************************************************
	* \brief Tells which ID correspond to which frame
	*	Tells which ID correspond to which frame, as given to Measure() and MeasureAndVisualize(),
	*	optionally can give name to an ID (default is empty name, L"").
	*	For instance, if it is metric with one ID without name and two frames, it should return ID, (0,1)
	*	and if it is with two IDs without names for two frames it should return ID1, (0), ID2, (1).
	*
	* \param visualize		[IN] - true if metric will be measured with visualization turned on.
	*						This parameter allows metric to have different amount of IDs when visualization
	*						is on or off.
	* \return vector of pairs <id, < vector of frame numbers corresponding to this id (for instance 0,1), name of ID>>.
	*/
	virtual vector< pair <IMetricPlugin::ID, pair< vector <int>, wstring> > >	MapIDToFrame(bool visualize) = 0;

	/**
	**************************************************************************
	* \brief Returns average result
	*
	* \param visualize		[IN] - whether we ask for result of measurement with visualization.
	* \return vector of pairs "id, average value corresponding to the ID".
	*/
	virtual	vector< pair <IMetricPlugin::ID, float> >			CalculateAverage(bool visualize) = 0;

	/**
	**************************************************************************
	* \brief Returns amount of video required for measurement (possibly with visualization)
	*
	* \param visualize		[IN] - if true, return amount of frames required for visualization.
	* \return amount of input frames.
	*/
	virtual int									GetVideoNum(bool visualize) = 0;

	/**
	**************************************************************************
	* \brief Returns supported color components
	* \return vector of supported color components.
	*/
	virtual vector <IMetricPlugin::COLORCOMPONENT> GetSupportedColorcomponents() = 0;

	/**
	**************************************************************************
	* \brief Returns name of metric
	* \return name that will be written in name of resulting CSV file.
	*		  Usually in small letters without spaces, like "psnr" of "blurring_measure".
	*/
	virtual wstring				GetName() = 0;

	/**
	**************************************************************************
	* \brief Returns name of metric that will be used in interface
	* \return name that will be used in interface. Usually in big letters like 
	*		"PSNR" of "Blurring Measure".
	*/
	virtual wstring				GetInterfaceName() = 0;

	/**
	**************************************************************************
	* \brief Returns units of metrics, for instance L"dB" or empty string L""
	* \return units of metrics, for instance L"dB" or empty string L"".
	*/
	virtual wstring				GetUnit() = 0;

	/**
	**************************************************************************
	* \return URL of information about metric.
	*/
	virtual wstring				GetMetrInfoURL() = 0;
	
	/**
	**************************************************************************
	* \brief Returns required width multiply
	* \return required width multiply; if metric takes any video, it must return 1.
	*/
	virtual int					GetWidthMultiply() = 0;

	/**
	**************************************************************************
	* \brief Returns required height multiply
	* \return required height multiply; if metric takes any video, it must return 1.
	*/
	virtual int					GetHeightMultiply() = 0;

	/**
	**************************************************************************
	* \brief Returns information about interpretation of metric values
	* \return true, if metric is the bigger the better. False if the smaller the better.
	*/
	virtual bool				GetMetrIncline() = 0;		
	
	/**
	**************************************************************************
	* \brief Returns true if metric can be configured
	* \return if metric can be configured with a string, then it must return true, 
	*	false otherwise.
	*/
	virtual bool				IsConfigurable() = 0;

	/**
	**************************************************************************
	* \brief Configurates metric with a given string
	*
	*	If metric returns true for IsConfigurable(), then this method is ALWAYS called after Init().
	*
	*	\param configuration_string [IN] - line that is used to configure metric in interface.
	*	\return true if configuration is supported and false if configuration is wrong.
	*/
	virtual bool				Config(const wstring &configuration_string) = 0;

	/**
	**************************************************************************
	* \brief Gets default configuration string
	*
	*	if metric returns true for IsConfigurable(), then this method is called to get
	*	its' default configuration line.
	*	\param def_config [OUT] - default configuration string. Returned string must be null-terminated.
	*/
	virtual void				GetDefaultConfiguration(wstring &def_config) = 0;
};